# -*- coding: utf-8 -*-
"""
    wtfgeo.fields
    =============

    :copyright: (c) 2015 by Erle Carrara.
    :license: BSD, see LICENSE for more details.
"""

import geojson
from shapely.geometry import shape
from geoalchemy2.shape import to_shape, from_shape
from wtforms.fields import Field
from wtfgeo.widgets import (GeometryEditor,
                            PolylineEditor,
                            PolygonEditor,
                            RectangleEditor,
                            CircleEditor,
                            PointEditor)


class GeometryField(Field):

    WGS84_SRID = 4326

    widget = GeometryEditor()

    def __init__(self, label='', validators=None, srid=WGS84_SRID,
                 only_one=True, widget_options=None, **kwargs):
        super(GeometryField, self).__init__(label, validators, **kwargs)
        self.data = None
        self.srid = srid
        self.only_one = only_one
        self.widget_options = widget_options or {}

    def __call__(self, **kwargs):
        if self.only_one:
            self.widget_options['max'] = 1

        return self.widget(self, options=self.widget_options, **kwargs)

    def _value(self):
        if self.data is not None:
            if not isinstance(self.data, list):
                self.data = [self.data]

            return geojson.dumps({
                'type': 'FeatureCollection',
                'features': [{
                    'type': 'Feature',
                    'properties': {},
                    'geometry': to_shape(wkb_element)
                } for wkb_element in self.data]
            })

        return u''

    def process_data(self, value):
        self.data = value

    def process_formdata(self, valuelist):
        if not valuelist:
            self.data = None
            return

        self.data = []
        geo = geojson.loads(valuelist[0])
        for feature in geo.features:
            if feature.geometry:
                self.data.append(from_shape(shape(feature.geometry)))

        if self.only_one:
            self.data = self.data[0] if len(self.data) > 0 else None


class PolylineField(GeometryField):
    widget = PolylineEditor()


class PolygonField(GeometryField):
    widget = PolygonEditor()


class RectangleField(GeometryField):
    widget = RectangleEditor()


class CircleField(GeometryField):
    widget = CircleEditor()


class PointField(GeometryField):
    widget = PointEditor()



# -*- coding: utf-8 -*-
"""
    wtfgeo.widgets
    ==============

    :copyright: (c) 2015 by Erle Carrara.
    :license: BSD, see LICENSE for more details.
"""

import json
from wtforms.widgets import Input, HTMLString


class GeometryEditor(Input):

    input_type = 'hidden'

    attribution = 'Map data &copy; <a href="http://openstreetmap.org">' \
                  'OpenStreetMap</a> contributors'

    shapes = ['polyline', 'rectangle', 'circle', 'marker']

    def _update_options(self, options):
        if options:
            for key, value in options.iteritems():
                if key in self.options and isinstance(self.options[key], dict):
                    self.options[key].update(value)
                else:
                    self.options[key] = value

    def __init__(self, **kwargs):
        self.options = {
            'toolbar': {
                'draw': {
                    'polyline': False,
                    'polygon': False,
                    'rectangle': False,
                    'circle': False,
                    'marker': False
                },
                'edit': {
                    'remove': {},
                    'edit': {}
                }
            },
            'center': None,
            'zoom': None,
            'max': None,
            'baselayer': {
                'url': 'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
                'options': {
                    'attribution': self.attribution
                }
            }
        }

        self._update_options(kwargs)

    def __call__(self, field, options=None, **kwargs):
        for shape in self.shapes:
            self.options['toolbar']['draw'][shape] = {}

        self._update_options(options)

        kwargs.setdefault('id', field.id)
        kwargs.setdefault('type', self.input_type)
        kwargs.setdefault('data-geometry-editor', True)
        kwargs.setdefault('data-options', json.dumps(self.options))

        if 'value' not in kwargs:
            kwargs['value'] = HTMLString(field._value())

        params = self.html_params(name=field.name, **kwargs)
        return HTMLString('<input {0}>'.format(params))


class PolylineEditor(GeometryEditor):
    shapes = ['polyline']


class PolygonEditor(GeometryEditor):
    shapes = ['polygon']


class RectangleEditor(GeometryEditor):
    shapes = ['rectangle']


class CircleEditor(GeometryEditor):
    shapes = ['circle']


class PointEditor(GeometryEditor):
    shapes = ['marker']



# -*- coding: utf-8 -*-
"""
WTForms-Geo
~~~~~~~~~~~
"""

from setuptools import setup, find_packages


setup(
    name='wtforms-geo',
    version='0.0.1',
    description='WTForms geojson fields and widgets.',
    long_description=__doc__,
    url='https://github.com/ecarrara/wtforms-geo',
    author='Erle Carrara',
    author_email='carrara.erle@gmail.com',
    license='BSD',
    zip_safe=False,
    platforms='any',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3'
    ],
    keywords='form geo gis',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=[
        'WTForms',
        'geojson',
        'shapely',
        'GeoAlchemy2',
    ]
)
